unit MEdit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, strutils, Controls, Graphics, Dialogs,
  StdCtrls, LazUTF8;

type

  { TMaskedEdit }

  TMaskedEdit = class(TEdit)
  private
    FMaskChar: Char;
    FMasked: Boolean;
    FMaskFull: Boolean;
    FMaskLen: Integer;
    FMaskStart: Integer;
    FUnmaskValue: String;
    procedure SetMaskChar(AValue: Char);
    procedure SetMasked(AValue: Boolean);
    procedure SetMaskFull(AValue: Boolean);
    procedure SetMaskLen(AValue: Integer);
    procedure SetMaskStart(AValue: Integer);
  protected
    procedure Change; override;
    procedure RealSetText(const AValue: TCaption); override;
  public
    constructor Create(TheOwner: TComponent); override;
    function GetUnmasked: String;
  published
    property MaskChar: Char read FMaskChar write SetMaskChar;
    property MaskStart: Integer read FMaskStart write SetMaskStart;
    property MaskLen: Integer read FMaskLen write SetMaskLen;
    property MaskFull: Boolean read FMaskFull write SetMaskFull;
    property Masked: Boolean read FMasked write SetMasked;
  private
    function MaskString(const Src: String): String;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('CustomEdit',[TMaskedEdit]);
end;

{ TMaskedEdit }
procedure TMaskedEdit.SetMaskChar(AValue: Char);
begin
  if FMaskChar = AValue then Exit;
  FMaskChar := AValue;
end;

procedure TMaskedEdit.SetMasked(AValue: Boolean);
begin
  if FMasked = AValue then Exit;
  FMasked := AValue;
  if (FMaskStart > 0) and (FMaskLen > 0) then
  if FMasked then
    Text := MaskString(Text)
  else
    Text := FUnmaskValue;
end;

procedure TMaskedEdit.SetMaskFull(AValue: Boolean);
begin
  if FMaskFull = AValue then Exit;
  FMaskFull := AValue;
  if MaskFull then
    Text := MaskString(Text)
  else
    Text := FUnmaskValue;
end;

procedure TMaskedEdit.SetMaskLen(AValue: Integer);
begin
  if FMaskLen = AValue then Exit;
  FMaskLen := AValue;
end;

procedure TMaskedEdit.SetMaskStart(AValue: Integer);
begin
  if FMaskStart = AValue then Exit;
  FMaskStart := AValue;
end;

procedure TMaskedEdit.Change;
begin
  if not Masked then FUnmaskValue := Text;
  inherited Change;
  Text := MaskString(Text);
end;

procedure TMaskedEdit.RealSetText(const AValue: TCaption);
begin
  FUnmaskValue := RealGetText;
  inherited RealSetText(AValue);
end;

constructor TMaskedEdit.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
end;

function TMaskedEdit.GetUnmasked: String;
begin
//  FUnmaskValue := RealGetText;
  Result := FUnmaskValue;
end;

function TMaskedEdit.MaskString(const Src: String): String;
var
  b, m, e: String;
begin
  if (not FMasked) then
  begin
    Result := Text;
  end;
  if FMaskFull then
  begin
    Result := Utf8AddCharR(FMaskChar, '', Length(Text));
  end;
  if FMasked then
  begin
    b := Utf8Copy(Src, 1, FMaskStart - 1);
    m := Utf8AddChar(FMaskChar, '', FMaskLen);
    e := Utf8Copy(Src, FMaskStart + FMaskLen, Length(Src));
    Result := b + m + e;
  end;
end;

end.
